/**
 * @file Dynamics.hpp
 * @author The ARTIS Development Team
 * See the AUTHORS or Authors.txt file
 */

/*
 * ARTIS - the multimodeling and simulation environment
 * This file is a part of the ARTIS environment
 *
 * Copyright (C) 2013-2019 ULCO http://www.univ-littoral.fr
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef DTSS_DYNAMICS
#define DTSS_DYNAMICS 1

#include <artis-star/common/Bag.hpp>
#include <artis-star/common/ExternalEvent.hpp>
#include <artis-star/common/Parameters.hpp>
#include <artis-star/common/States.hpp>
#include <artis-star/kernel/dtss/Simulator.hpp>

namespace artis {
    namespace dtss {

        template<class Time, class Dyn,
                class Parameters = common::NoParameters>
        class Dynamics : public common::States<Time, Dyn> {
            typedef dtss::Simulator<Time, Dyn, Parameters> Simulator;

        public:
            Dynamics(const std::string& name, const Context<Time, Dyn, Parameters>& context)
                    :
                    _name(name), _simulator(context.simulator()) { }

            virtual ~Dynamics() { }

            virtual void transition(const common::Bag<Time>& /* x */,
                    typename Time::type /* t */) { }

            virtual typename Time::type
            start(typename Time::type/* time */) { return Time::infinity; }

            virtual common::Bag<Time> lambda(
                    typename Time::type /* time */) const { return common::Bag<Time>(); }

            virtual common::Value observe(const typename Time::type& /* t */,
                    unsigned int /* index */) const { return common::Value(); }

            const std::string& get_name() const { return _name; }

            void input_port(common::Port p)
            {
                _simulator->add_in_port(p);
            }

            void input_ports(std::initializer_list<common::Port> list)
            {
                for (typename std::initializer_list<common::Port>::iterator it =
                        list.begin(); it != list.end(); ++it) {
                    _simulator->add_in_port(*it);
                }
            }

            void output_port(common::Port p)
            {
                _simulator->add_out_port(p);
            }

            void output_ports(std::initializer_list<common::Port> list)
            {
                for (typename std::initializer_list<common::Port>::iterator it =
                        list.begin(); it != list.end(); ++it) {
                    _simulator->add_out_port(*it);
                }
            }

            void restore(const common::context::State<Time>& state)
            {
                common::States<Time, Dyn>::restore(static_cast<Dyn*>(this), state);
            }

            void save(common::context::State<Time>& state) const
            {
                common::States<Time, Dyn>::save(static_cast<const Dyn*>(this), state);
            }

        private:
            std::string _name;
            Simulator* _simulator;
        };

    }
} // namespace artis dtss

#endif
